# :nodoc:
#
# Author:: Nathaniel Talbott.
# Copyright:: Copyright (c) 2000-2002 Nathaniel Talbott. All rights reserved.
# License:: Ruby license.

require 'fox'
require 'test/unit/ui/testrunnermediator'
require 'test/unit/ui/testrunnerutilities'

include Fox

module Test
	module Unit
		module UI
			module Fox # :nodoc:

				# Runs a Test::Unit::TestSuite in a Fox UI. Obviously,
				# this one requires you to have Fox
				# (http://www.fox-toolkit.org/fox.html) and the Ruby
				# Fox extension (http://fxruby.sourceforge.net/)
				# installed.
				class TestRunner < FXMainWindow

					extend TestRunnerUtilities
	
					# Creates a new TestRunner and runs the suite.
					def TestRunner.run(suite)
						new(suite).start
					end

					# Creates a new TestRunner for running the passed
					# suite.
					def initialize(suite)
						# Initialize base class first
						super(main_app, "Test::Unit TestRunner", nil, nil, DECOR_ALL, 0, 0, 450)

						FXTooltip.new(main_app)

						if (suite.respond_to?(:suite))
							@suite = suite.suite
						else
							@suite = suite
						end

						@red = false
						status_entry
						main_panel
					end
	
					# Begins the test run.
					def start
						setup_ui
						setup_mediator
						attach_to_mediator
						start_ui
					end

					def setup_mediator # :nodoc:
						@mediator = TestRunnerMediator.new(@suite)
						suite_name = @suite.to_s
						if ( @suite.kind_of?(Module) )
							suite_name = @suite.name
						end
						suite_name_entry.text = suite_name
					end
					def setup_ui # :nodoc:
						@red = false
					end
					def attach_to_mediator # :nodoc:
						@mediator.add_listener(TestRunnerMediator::RESET, &method(:reset_ui))
						@mediator.add_listener(TestRunnerMediator::FAULT_ADDED, &method(:add_fault))
						@mediator.add_listener(TestRunnerMediator::RESULT_CHANGED, &method(:result_changed))
						@mediator.add_listener(TestRunnerMediator::STATUS_CHANGED, &method(:update_status))
					end
	
					def create # :nodoc:
						super
						show(PLACEMENT_SCREEN)
					end
	
					def start_ui # :nodoc:
						main_app.create
						main_app.run
					end
					def stop # :nodoc:
						main_app.exit(0)
					end
					def reset_ui(count) # :nodoc:
						test_progress_bar.barColor = green_style
						test_progress_bar.total = count
						test_progress_bar.progress = 0
						@red = false
	
						run_count_label.text = "0"
						assertion_count_label.text = "0"
						failure_count_label.text = "0"
						error_count_label.text = "0"
	
						fault_list.clearItems
					end
					def add_fault(fault) # :nodoc:
						if ( ! @red )
							test_progress_bar.barColor = red_style
							@red = true
						end
						item = FaultListItem.new(fault)
						fault_list.appendItem(item)
					end
					def show_fault(fault) # :nodoc:
						raw_show_fault(fault.long_display)
					end
					def raw_show_fault(string) # :nodoc:
						detail_text.setText(string)
					end
					def clear_fault # :nodoc:
						raw_show_fault("")
					end
					def result_changed(result) # :nodoc:
						test_progress_bar.progress = result.run_count
	
						run_count_label.text = result.run_count.to_s
						assertion_count_label.text = result.assertion_count.to_s
						failure_count_label.text = result.failure_count.to_s
						error_count_label.text = result.error_count.to_s

						 # repaint now!
						 @info_panel.repaint
						 getApp().flush
					end
					def update_status(status) # :nodoc:
						status_entry.statusline.text = status.message
					end
	
					def main_app # :nodoc:
						lazy_initialize(:main_app) {
							@main_app = FXApp.new("TestRunner", "Test::Unit")
							@main_app.init([])
						}
					end
	
					def main_panel # :nodoc:
						lazy_initialize(:main_panel) {
							@main_panel = FXVerticalFrame.new(self, LAYOUT_FILL_X | LAYOUT_FILL_Y)
							@main_panel.vSpacing = 10
							suite_panel
							test_progress_bar
							info_panel
							list_panel
							detail_panel
						}
					end
	
					def suite_panel # :nodoc:
						lazy_initialize(:suite_panel) {
							@suite_panel = FXHorizontalFrame.new(@main_panel, LAYOUT_SIDE_LEFT | LAYOUT_FILL_X)
							FXLabel.new(@suite_panel, "Suite:")
							suite_name_entry
							run_button
						}
					end
					def suite_name_entry # :nodoc:
						lazy_initialize(:suite_name_entry) {
							@suite_name_entry = FXTextField.new(@suite_panel, 30, nil, 0, TEXTFIELD_NORMAL | LAYOUT_FILL_X)
							@suite_name_entry.disable
						}
					end
					def run_button # :nodoc:
						lazy_initialize(:run_button) {
							@run_button = FXButton.new(@suite_panel, "&Run\tRun the current suite").connect(SEL_COMMAND) { @mediator.run_suite }
						}
					end
					def test_progress_bar # :nodoc:
						lazy_initialize(:test_progress_bar) {
							@test_progress_bar = FXProgressBar.new(@main_panel, nil, 0, PROGRESSBAR_NORMAL | LAYOUT_FILL_X)
							@test_progress_bar.total = @suite.size
							@test_progress_bar.barColor = green_style
						}
					end
					def green_style # :nodoc:
						lazy_initialize(:green_style) {
							@green_style = FXRGBA(0,0xFF,0,0xFF) #0x00FF0000
						}
					end
					def red_style # :nodoc:
						lazy_initialize(:red_style) {
							@red_style = FXRGBA(0xFF,0,0,0xFF) #0xFF000000
						}
					end
					def info_panel # :nodoc:
						lazy_initialize(:info_panel) {
							@info_panel = FXMatrix.new(@main_panel, 1, MATRIX_BY_ROWS | LAYOUT_FILL_X)
							FXLabel.new(@info_panel, "Runs:", nil, JUSTIFY_CENTER_X | LAYOUT_FILL_COLUMN)
							run_count_label
							FXLabel.new(@info_panel, "Assertions:", nil, JUSTIFY_CENTER_X | LAYOUT_FILL_COLUMN)
							assertion_count_label
							FXLabel.new(@info_panel, "Failures:", nil, JUSTIFY_CENTER_X | LAYOUT_FILL_COLUMN)
							failure_count_label
							FXLabel.new(@info_panel, "Errors:", nil, JUSTIFY_CENTER_X | LAYOUT_FILL_COLUMN)
							error_count_label
						}
					end
					def run_count_label # :nodoc:
						lazy_initialize(:run_count_label) {
							@run_count_label = FXLabel.new(@info_panel, "0", nil, JUSTIFY_CENTER_X | LAYOUT_FILL_COLUMN)
						}
					end
					def assertion_count_label # :nodoc:
						lazy_initialize(:assertion_count_label) {
							@assertion_count_label = FXLabel.new(@info_panel, "0", nil, JUSTIFY_CENTER_X | LAYOUT_FILL_COLUMN)
						}
					end
					def failure_count_label # :nodoc:
						lazy_initialize(:failure_count_label) {
							@failure_count_label = FXLabel.new(@info_panel, "0", nil, JUSTIFY_CENTER_X | LAYOUT_FILL_COLUMN)
						}
					end
					def error_count_label # :nodoc:
						lazy_initialize(:error_count_label) {
							@error_count_label = FXLabel.new(@info_panel, "0", nil, JUSTIFY_CENTER_X | LAYOUT_FILL_COLUMN)
						}
					end
					def list_panel # :nodoc:
						lazy_initialize(:list_panel) {
							@list_panel = FXHorizontalFrame.new(@main_panel, LAYOUT_FILL_X | FRAME_SUNKEN | FRAME_THICK)
							fault_list
						}
					end
					def fault_list # :nodoc:
						lazy_initialize(:fault_list) {
							@fault_list = FXList.new(@list_panel, 10, nil, 0, LIST_SINGLESELECT | LAYOUT_FILL_X) #, 0, 0, 0, 150)
							@fault_list.connect(SEL_COMMAND) do |sender, sel, ptr|
								if sender.retrieveItem(sender.currentItem).selected?
									show_fault(sender.retrieveItem(sender.currentItem).fault)
								else
									clear_fault
								end
							end
						}
					end
					def detail_panel # :nodoc:
						lazy_initialize(:detail_panel) {
							@detail_panel = FXHorizontalFrame.new(@main_panel, LAYOUT_FILL_X | LAYOUT_FILL_Y | FRAME_SUNKEN | FRAME_THICK)
							detail_text
						}
					end
					def detail_text # :nodoc:
						lazy_initialize(:detail_text) {
							@detail_text = FXText.new(@detail_panel, nil, 0, TEXT_READONLY | LAYOUT_FILL_X | LAYOUT_FILL_Y)
							class << @detail_text # :nodoc:
								def setText(text)
									super(text.gsub(/\n\t/, "\n" + (" " * 4)))
								end
							end
						}
					end
					def status_entry # :nodoc:
						lazy_initialize(:status_entry) {
							@status_entry = FXStatusbar.new(self, LAYOUT_SIDE_BOTTOM | LAYOUT_FILL_X)
						}
					end
	
					def lazy_initialize(symbol) # :nodoc:
						if ( instance_eval("@" + symbol.to_s).nil? )
							yield
						end
						return instance_eval("@" + symbol.to_s)
					end
				end
	
				class FaultListItem < FXListItem # :nodoc: all
					attr_reader(:fault)
					def initialize(fault)
						super(fault.short_display)
						@fault = fault
					end
				end
			end
		end
	end
end

if __FILE__ == $0
	Test::Unit::UI::Fox::TestRunner.start_command_line_test
end
